<?php

/**
 * Plugin Name: Google Consent Mode
 * Description: Intégration de Google Consent Mode v2 pour WordPress, permettant une gestion dynamique du consentement des utilisateurs.
 * Version: 1.0.2
 * Author: Slaaap
 */

/**
 * Active les mises à jour automatiques pour ce plugin.
 */
function slaaap_auto_update_plugin($update, $item) {
    // Spécifiez les slugs des plugins que vous voulez mettre à jour automatiquement
    $auto_update_plugins = array(
        'google-consent-mode',
    );

    if (in_array($item->slug, $auto_update_plugins)) {
        return true;
    }

    return $update;
}
add_filter('auto_update_plugin', 'slaaap_auto_update_plugin', 10, 2);


/**
 * Vérifie si une mise à jour du plugin est disponible.
 */
function slaaap_check_for_plugin_update($checked_data) {
    global $wp_version;

    $api_url = 'https://wordpress.slaaap.fr/plugins/google-consent-mode/update-check.php';

    $request_args = array(
        'slug' => 'google-consent-mode',
        'version' => $checked_data->checked['google-consent-mode/google-consent-mode.php'],
    );

    $response = wp_remote_post($api_url, array(
        'body' => json_encode($request_args),
        'user-agent' => 'WordPress/' . $wp_version,
    ));

    if (!is_wp_error($response) && wp_remote_retrieve_response_code($response) == 200) {
        $update_data = json_decode(wp_remote_retrieve_body($response));
        if ($update_data && property_exists($update_data, 'new_version')) {
            if (version_compare($update_data->new_version, $checked_data->checked['google-consent-mode/google-consent-mode.php'], '>')) {
                $checked_data->response['google-consent-mode/google-consent-mode.php'] = $update_data;
            }
        }
    }

    return $checked_data;
}
add_filter('pre_set_site_transient_update_plugins', 'slaaap_check_for_plugin_update');


/**
 * Gère les appels API pour récupérer les informations de mise à jour du plugin.
 */
function slaaap_plugin_api_call($def, $action, $args) {
    global $wp_version;

    $api_url = 'https://s2.slaaap.fr/plugins-wordpress/google-consent-mode/update-info.php';

    if ($action == 'plugin_information' && isset($args->slug) && $args->slug == 'google-consent-mode') {
        $response = wp_remote_post($api_url, array(
            'body' => json_encode(array('slug' => 'google-consent-mode')),
            'user-agent' => 'WordPress/' . $wp_version,
        ));

        if (!is_wp_error($response) && wp_remote_retrieve_response_code($response) == 200) {
            $plugin_info = json_decode(wp_remote_retrieve_body($response));
            return $plugin_info;
        }
    }

    return $def;
}
add_filter('plugins_api', 'slaaap_plugin_api_call', 10, 3);


/**
 * Ajoute un script personnalisé au header pour gérer dynamiquement le consentement des utilisateurs.
 * Ce script initialise Google Consent Mode et écoute les interactions des utilisateurs avec les boutons de consentement.
 */
function add_custom_consent_script() {
?>
    <script type="text/javascript">
        window.dataLayer = window.dataLayer || [];

        function gtag() {
            dataLayer.push(arguments);
        }

        // Initialisation de Google Consent Mode avec valeurs par défaut.
        function initializeConsent() {
            var consent = localStorage.getItem('userConsent');
            if (consent) {
                var consentObj = JSON.parse(consent);
                gtag("consent", "default", {
                    analytics_storage: consentObj.analytics_storage || "denied",
                    ad_storage: consentObj.ad_storage || "denied",
                    ad_user_data: consentObj.ad_user_data || "denied",
                    ad_personalization: consentObj.ad_personalization || "denied",
                    personalization_storage: consentObj.personalization_storage || "denied",
                    functionality_storage: consentObj.functionality_storage || "denied",
                    security_storage: consentObj.security_storage || "denied",
                    wait_for_update: 500
                });
            } else {
                gtag("consent", "default", {
                    analytics_storage: "denied",
                    ad_storage: "denied",
                    ad_user_data: "denied",
                    ad_personalization: "denied",
                    personalization_storage: "denied",
                    functionality_storage: "denied",
                    security_storage: "denied",
                    wait_for_update: 500
                });
            }
            gtag("set", "ads_data_redaction", true);
        }


        initializeConsent();

        document.addEventListener("DOMContentLoaded", function() {

            // Vérification immédiate de la présence des boutons de consentement
            var acceptButtonPresent = document.getElementById('accept-cookies') !== null;
            var rejectButtonPresent = document.getElementById('reject-cookies') !== null;

            if (!acceptButtonPresent || !rejectButtonPresent) {
                console.error("Erreur : Les éléments de consentement 'accept-cookies' ou 'reject-cookies' ne sont pas présents sur la page.");
            }

            document.addEventListener('click', function(event) {
                // Vérifie si l'élément cliqué est le lien d'acceptation des cookies.
                var isAcceptButton = event.target.closest('#accept-cookies');
                if (isAcceptButton) {
                    updateConsent(true);
                }

                // Vérifie si l'élément cliqué est le lien de refus des cookies.
                var isRejectButton = event.target.closest('#reject-cookies');
                if (isRejectButton) {
                    updateConsent(false);
                }
            });

            function updateConsent(consent) {
				// Mise à jour de tous les paramètres de consentement en fonction du choix de l'utilisateur
				var consentSettings = {
					analytics_storage: consent ? "granted" : "denied",
					ad_storage: consent ? "granted" : "denied",
					ad_user_data: consent ? "granted" : "denied",
					ad_personalization: consent ? "granted" : "denied",
					personalization_storage: consent ? "granted" : "denied",
					functionality_storage: consent ? "granted" : "denied",
					security_storage: consent ? "granted" : "denied"
				};
				// Mise à jour de Google Consent Mode avec les nouveaux paramètres de consentement
				gtag("consent", "update", consentSettings);
				// Stockage des préférences de consentement dans le stockage local
				localStorage.setItem('userConsent', JSON.stringify(consentSettings));
				// Log dans la console pour le débogage
				console.log("Consentement mis à jour :", consent ? "Accepté" : "Refusé", consentSettings);
			}

        });
    </script>
<?php
}
add_action('wp_head', 'add_custom_consent_script', 1);
